from .SmartDevice import SmartDevice
from .SmartLight import SmartLight
from .SmartVacuum import SmartVacuum


TYPES = {
    "devices.types.light": SmartLight,
    "devices.types.vacuum_cleaner": SmartVacuum,
}



from YandexIOT.Devices import SmartDevice
from YandexIOT.Devices.SmartDevice import UnsupportedFeature


class SmartVacuum(SmartDevice):
    def __init__(self, device_json, home):
        super(SmartVacuum, self).__init__(device_json, home)

    def turn_on(self):
        """Turn on the vacuum.

        :return: SmartDeviceResponse
        """

        return self._change_state(True)

    def _change_state(self, state):
        if "devices.capabilities.on_off" not in self.get_capabilities():
            raise UnsupportedFeature(f"Device '{self.get_name()}' does not support turning it on/off.")

        return self._send_actions([
            {
                "type": "devices.capabilities.on_off",
                "state": {
                    "instance": "on",
                    "value": state
                }
            }
        ])



import requests


class SmartDeviceResponse:
    def __init__(self, response_json):
        self.__response_json = response_json

    def get_status_str(self):
        return self.__response_json['status']

    def get_status(self):
        return self.__response_json['status'] == "ok"

    def get_message(self):
        return "" if "message" not in self.__response_json else self.__response_json['message']


class SmartDevice:
    def __init__(self, device_json, home):
        self.__device_json = device_json
        self.__home = home

        self._capabilities = []

        for i in self.__device_json['capabilities']:
            self._capabilities.append(i['type'])

    def _get_home(self):
        return self.__home

    def _device_json(self):
        return self.__device_json

    def _send_actions(self, actions: list):
        req = requests.post("https://api.iot.yandex.net/v1.0/devices/actions",
                            headers=self._get_home().headers,
                            json={
                                "devices": [{
                                    "id": self.get_id(),
                                    "actions": actions
                                }]
                            })
        return SmartDeviceResponse(req.json())

    def get_capabilities(self):
        return self._capabilities

    def get_id(self):
        return self.__device_json['id']

    def get_name(self):
        return self.__device_json['name']

    def get_type(self):
        return self.__device_json['type']


class UnsupportedFeature(Exception):
    pass



import YandexIOT.Devices
import typing as t
import requests


class SmartHome:
    def __init__(self, token: str, token_type="Bearer"):
        """Initializer for SmartHome class.

        :param: token: Yandex oauth 2.0 token with access to SmartHome control.
        :param: token_type: type of the token that Yandex oauth service has returned.
        """

        self.token = token
        self.token_type = token_type

        self.headers = {
            "Authorization": f"{self.token_type} {self.token}"
        }

    def get_devices(self) -> t.List[YandexIOT.Devices.SmartDevice]:
        """Returns a list of currently available devices.
        """
        
        info = self._get_home_info()
        devices = []

        if info[0]:
            for device in info[1]['devices']:
                if device['type'] not in YandexIOT.Devices.TYPES:
                    continue

                device_class = YandexIOT.Devices.TYPES[device['type']](device, self)
                devices.append(device_class)

        return devices

    def get_device_by_name(self, device_name):
        """Returns a device by its name.
        """
        
        device = None
        for i in self.get_devices():
            if i.get_name() == device_name:
                device = i
                break

        return device

    def get_device_by_id(self, device_id):
        """Returns a device by its id.
        """
        
        device = None
        for i in self.get_devices():
            if i.get_id() == device_id:
                device = i
                break

        return device

    def _get_home_info(self) -> t.Tuple[bool, dict]:
        req = requests.get("https://api.iot.yandex.net/v1.0/user/info", headers=self.headers)

        data = None
        result = req.status_code == 200

        if result:
            data = req.json()

        return result, data


import colorsys
from YandexIOT.Devices import SmartDevice
from YandexIOT.Devices.SmartDevice import UnsupportedFeature


class SmartLight(SmartDevice):
    def __init__(self, device_json, home):
        super(SmartLight, self).__init__(device_json, home)

    def turn_on(self):
        """Turn on the light.

        :return: SmartDeviceResponse
        """

        return self._change_state(True)

    def turn_off(self):
        """Turn off the light.

        :return: SmartDeviceResponse
        """

        return self._change_state(False)

    def set_brightness(self, value):
        """Change the brightness level of the light.

        :param value: A value from 1 to 100.
        :return: SmartDeviceResponse
        """

        return self._change_brightness(value)

    def set_color(self, color):
        """Change the color of the light.

        :param color: A tuple of 3 RGB integers.
        :return: SmartDeviceResponse
        """

        return self._change_color(color)

    def set_scene(self, scene):
        """Change the scene of the light.

        :param scene: Scene type.
        :return: SmartDeviceResponse
        """

        return self._change_scene(scene)

    def _change_state(self, state):
        if "devices.capabilities.on_off" not in self.get_capabilities():
            raise UnsupportedFeature(f"Device '{self.get_name()}' does not support turning it on/off.")

        return self._send_actions([
            {
                "type": "devices.capabilities.on_off",
                "state": {
                    "instance": "on",
                    "value": state
                }
            }
        ])

    def _change_brightness(self, value):
        if "devices.capabilities.range" not in self.get_capabilities():
            raise UnsupportedFeature(f"Device '{self.get_name()}' does not support changing its brightness.")

        return self._send_actions([
            {
                "type": "devices.capabilities.range",
                "state": {
                    "instance": "brightness",
                    "value": value
                }
            }
        ])

    def _change_color(self, color):
        if "devices.capabilities.color_setting" not in self.get_capabilities():
            raise UnsupportedFeature(f"Device '{self.get_name()}' does not support changing its color.")

        value = None
        color_model = self.get_device_color_model()

        if color_model == "rgb":
            value = int('%02x%02x%02x' % color, 16)
        elif color_model == "hsv":
            (h, s, v) = colorsys.rgb_to_hsv(color[0] / 255, color[1] / 255, color[2] / 255)
            value = {
                "h": int(h * 360),
                "s": int(s * 100),
                "v": int(v * 100),
            }

        return self._send_actions([
            {
                "type": "devices.capabilities.color_setting",
                "state": {
                    "instance": color_model,
                    "value": value
                }
            }
        ])

    def _change_scene(self, scene):
        if "devices.capabilities.color_setting" not in self.get_capabilities():
            raise UnsupportedFeature(f"Device '{self.get_name()}' does not support changing its scene.")

        if scene not in self.get_scenes():
            raise InvalidLightScene(f"Invalid scene '{scene}' for '{self.get_name()}'. "
                                    f"Try to call SmartLight.get_scenes() to get available scenes.")

        return self._send_actions([
            {
                "type": "devices.capabilities.color_setting",
                "state": {
                    "instance": "scene",
                    "value": scene
                }
            }
        ])

    def get_device_color_model(self):
        model = "rgb"
        for i in self._device_json()["capabilities"]:
            if i['type'] == "devices.capabilities.color_setting":
                model = i['parameters']['color_model']
                break
        return model

    def get_scenes(self):
        for i in self._device_json()["capabilities"]:
            if 'color_scene' in i['parameters']:
                return [j['id'] for j in i['parameters']['color_scene']['scenes']]

        return []


class InvalidLightScene(Exception):
    pass



from setuptools import find_packages, setup

setup(name='YandexIOT',
      version='0.1',
      description='Library for controlling Yandex Smart Devices in Python',
      url='https://github.com/Artingl/YandexIOT',
      author='Artingl',
      author_email='mikrobamboni@yandex.ru',
      packages=['YandexIOT', 'YandexIOT.Devices'],
      install_requires=[
          'requests==2.27.1',
      ])
